@@ -82,18 +82,22 @@ func Main() {
 			string(compress.CompressionTypeS2), string(compress.CompressionTypePGZIP),
 			string(compress.CompressionTypeZstandard),
 		)
-	backupCmd.Flag("type", fmt.Sprintf("backup type: <%s>/<%s>/<%s>", pbm.PhysicalBackup, pbm.LogicalBackup, pbm.IncrementalBackup)).
+	backupCmd.Flag("type",
+		fmt.Sprintf("backup type: <%s>/<%s>/<%s>/<%s>",
+			pbm.PhysicalBackup, pbm.LogicalBackup, pbm.IncrementalBackup, pbm.ExternalBackup)).
 		Default(string(pbm.LogicalBackup)).Short('t').
 		EnumVar(&backup.typ,
 			string(pbm.PhysicalBackup),
 			string(pbm.LogicalBackup),
 			string(pbm.IncrementalBackup),
+			string(pbm.ExternalBackup),
 		)
 	backupCmd.Flag("base", "Is this a base for incremental backups").BoolVar(&backup.base)
 	backupCmd.Flag("compression-level", "Compression level (specific to the compression type)").
 		IntsVar(&backup.compressionLevel)
 	backupCmd.Flag("ns", `Namespaces to backup (e.g. "db.*", "db.collection"). If not set, backup all ("*.*")`).StringVar(&backup.ns)
 	backupCmd.Flag("wait", "Wait for the backup to finish").Short('w').BoolVar(&backup.wait)
+	backupCmd.Flag("list-files", "Wait for the backup to finish").Short('l').BoolVar(&backup.externList)
 
 	cancelBcpCmd := pbmCmd.Command("cancel-backup", "Cancel backup")
 
@@ -101,6 +105,15 @@ func Main() {
 	descBcp := descBcp{}
 	descBcpCmd.Arg("backup_name", "Backup name").StringVar(&descBcp.name)
 
+	finishBackupName := ""
+	backupFinishCmd := pbmCmd.Command("backup-finish", "Finish external backup")
+	backupFinishCmd.Arg("backup_name", "Backup name").StringVar(&finishBackupName)
+
+	finishRestore := descrRestoreOpts{}
+	restoreFinishCmd := pbmCmd.Command("restore-finish", "Finish external backup")
+	restoreFinishCmd.Arg("restore_name", "Restore name").StringVar(&finishRestore.restore)
+	restoreFinishCmd.Flag("config", "Path to PBM config").Short('c').Required().StringVar(&finishRestore.cfg)
+
 	restoreCmd := pbmCmd.Command("restore", "Restore backup")
 	restore := restoreOpts{}
 	restoreCmd.Arg("backup_name", "Backup name to restore").StringVar(&restore.bcp)
@@ -108,6 +121,9 @@ func Main() {
 	restoreCmd.Flag("base-snapshot", "Override setting: Name of older snapshot that PITR will be based on during restore.").StringVar(&restore.pitrBase)
 	restoreCmd.Flag("ns", `Namespaces to restore (e.g. "db1.*,db2.collection2"). If not set, restore all ("*.*")`).StringVar(&restore.ns)
 	restoreCmd.Flag("wait", "Wait for the restore to finish.").Short('w').BoolVar(&restore.wait)
+	restoreCmd.Flag("external", "External restore.").Short('x').BoolVar(&restore.extern)
+	restoreCmd.Flag("config", "Mongod config for the source data. External backups only!").Short('c').StringVar(&restore.conf)
+	restoreCmd.Flag("ts", "MongoDB cluster time to restore to. In <T,I> format (e.g. 1682093090,9). External backups only!").StringVar(&restore.ts)
 	restoreCmd.Flag(RSMappingFlag, RSMappingDoc).Envar(RSMappingEnvVar).StringVar(&restore.rsMap)
 
 	replayCmd := pbmCmd.Command("oplog-replay", "Replay oplog")
@@ -153,7 +169,7 @@ func Main() {
 	logsCmd.Flag("tail", "Show last N entries, 20 entries are shown by default, 0 for all logs").Short('t').Default("20").Int64Var(&logs.tail)
 	logsCmd.Flag("node", "Target node in format replset[/host:posrt]").Short('n').StringVar(&logs.node)
 	logsCmd.Flag("severity", "Severity level D, I, W, E or F, low to high. Choosing one includes higher levels too.").Short('s').Default("I").EnumVar(&logs.severity, "D", "I", "W", "E", "F")
-	logsCmd.Flag("event", "Event in format backup[/2020-10-06T11:45:14Z]. Events: backup, restore, cancelBackup, resync, pitr, pitrestore, delete").Short('e').StringVar(&logs.event)
+	logsCmd.Flag("event", "Event in format backup[/2020-10-06T11:45:14Z]. Events: backup, restore, cancelBackup, resync, pitr, delete").Short('e').StringVar(&logs.event)
 	logsCmd.Flag("opid", "Operation ID").Short('i').StringVar(&logs.opid)
 	logsCmd.Flag("timezone", "Timezone of log output. `Local`, `UTC` or a location name corresponding to a file in the IANA Time Zone database, such as `America/New_York`").StringVar(&logs.location)
 	logsCmd.Flag("extra", "Show extra data in text format").Hidden().Short('x').BoolVar(&logs.extr)
@@ -202,7 +218,8 @@ func Main() {
 
 	var pbmClient *pbm.PBM
 	// we don't need pbm connection if it is `pbm describe-restore -c ...`
-	if describeRestoreOpts.cfg == "" {
+	// or `pbm restore-finish `
+	if describeRestoreOpts.cfg == "" && finishRestore.cfg == "" {
 		pbmClient, err = pbm.New(ctx, *mURL, "pbm-ctl")
 		if err != nil {
 			exitErr(errors.Wrap(err, "connect to mongodb"), pbmOutF)
@@ -218,6 +235,10 @@ func Main() {
 		out, err = runBackup(pbmClient, &backup, pbmOutF)
 	case cancelBcpCmd.FullCommand():
 		out, err = cancelBcp(pbmClient)
+	case backupFinishCmd.FullCommand():
+		out, err = runFinishBcp(pbmClient, finishBackupName)
+	case restoreFinishCmd.FullCommand():
+		out, err = runFinishRestore(finishRestore)
 	case descBcpCmd.FullCommand():
 		out, err = describeBackup(pbmClient, &descBcp)
 	case restoreCmd.FullCommand():
