@@ -1,19 +1,25 @@
 package cli
 
 import (
+	"bytes"
 	"context"
+	"encoding/json"
 	"fmt"
+	"io"
 	"io/ioutil"
+	"os"
 	"strconv"
 	"strings"
 	"time"
 
+	"github.com/mongodb/mongo-tools/common/db"
 	"github.com/pkg/errors"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"gopkg.in/yaml.v2"
 
 	"github.com/percona/percona-backup-mongodb/pbm"
 	"github.com/percona/percona-backup-mongodb/pbm/log"
+	"github.com/percona/percona-backup-mongodb/pbm/storage"
 )
 
 type restoreOpts struct {
@@ -21,8 +27,11 @@ type restoreOpts struct {
 	pitr     string
 	pitrBase string
 	wait     bool
+	extern   bool
 	ns       string
 	rsMap    string
+	conf     string
+	ts       string
 }
 
 type restoreRet struct {
@@ -41,9 +50,9 @@ func (r restoreRet) HasError() bool {
 func (r restoreRet) String() string {
 	switch {
 	case r.done:
-		m := "\nRestore successfully finished!\n"
+		m := fmt.Sprintf("\nRestore finished! Check pbm describe-restore %s", r.Name)
 		if r.physical {
-			m += "Restart the cluster and pbm-agents, and run `pbm config --force-resync`"
+			m += " -c </path/to/pbm.conf.yaml>\nRestart the cluster and pbm-agents, and run `pbm config --force-resync`"
 		}
 		return m
 	case r.err != "":
@@ -66,6 +75,21 @@ No other pbm command is available while the restore is running!
 	}
 }
 
+type externRestoreRet struct {
+	Name     string `json:"name,omitempty"`
+	Snapshot string `json:"snapshot,omitempty"`
+}
+
+func (r externRestoreRet) String() string {
+	return fmt.Sprintf(`
+	Ready to copy data to the nodes data directory.
+	After the copy is done, run: pbm restore-finish %s -c </path/to/pbm.conf.yaml>
+	Check restore status with: pbm describe-restore %s -c </path/to/pbm.conf.yaml>
+	No other pbm command is available while the restore is running!
+	`,
+		r.Name, r.Name)
+}
+
 func runRestore(cn *pbm.PBM, o *restoreOpts, outf outFormat) (fmt.Stringer, error) {
 	nss, err := parseCLINSOption(o.ns)
 	if err != nil {
@@ -87,57 +111,44 @@ func runRestore(cn *pbm.PBM, o *restoreOpts, outf outFormat) (fmt.Stringer, erro
 	}
 	tdiff := time.Now().Unix() - int64(clusterTime.T)
 
-	switch {
-	case o.bcp != "":
-		m, err := restore(cn, o.bcp, nss, rsMap, outf)
-		if err != nil {
-			return nil, err
-		}
-		if !o.wait {
-			return restoreRet{
-				Name:     m.Name,
-				Snapshot: o.bcp,
-				physical: m.Type == pbm.PhysicalBackup || m.Type == pbm.IncrementalBackup,
-			}, nil
-		}
-
-		typ := " logical restore.\nWaiting to finish"
-		if m.Type == pbm.PhysicalBackup {
-			typ = " physical restore.\nWaiting to finish"
-		}
-		fmt.Printf("Started%s", typ)
-		err = waitRestore(cn, m, tdiff)
-		if err == nil {
-			return restoreRet{
-				done:     true,
-				physical: m.Type == pbm.PhysicalBackup || m.Type == pbm.IncrementalBackup,
-			}, nil
-		}
-
-		if serr, ok := err.(errRestoreFailed); ok {
-			return restoreRet{err: serr.Error()}, nil
-		}
-		return restoreRet{err: fmt.Sprintf("%s.\n Try to check logs on node %s", err.Error(), m.Leader)}, nil
-	case o.pitr != "":
-		m, err := pitrestore(cn, o.pitr, o.pitrBase, nss, rsMap, outf)
-		if err != nil {
-			return nil, err
-		}
-		if !o.wait {
-			return restoreRet{PITR: o.pitr, Name: m.Name}, nil
-		}
-		fmt.Print("Started.\nWaiting to finish")
-		err = waitRestore(cn, m, tdiff)
-		if err != nil {
-			return restoreRet{err: err.Error()}, nil
-		}
-		return restoreRet{
-			done: true,
-			PITR: o.pitr,
-		}, nil
-	default:
-		return nil, errors.New("undefined restore state")
+	m, err := restore(cn, o, nss, rsMap, outf)
+	if err != nil {
+		return nil, err
 	}
+	if o.extern && outf == outText {
+		err = waitRestore(cn, m, pbm.StatusCopyReady, tdiff)
+		if err != nil {
+			return nil, errors.Wrap(err, "waiting for the `copyReady` status")
+		}
+
+		return externRestoreRet{Name: m.Name, Snapshot: o.bcp}, nil
+	}
+	if !o.wait {
+		return restoreRet{
+			Name:     m.Name,
+			Snapshot: o.bcp,
+			physical: m.Type == pbm.PhysicalBackup || m.Type == pbm.IncrementalBackup,
+		}, nil
+	}
+
+	typ := " logical restore.\nWaiting to finish"
+	if m.Type == pbm.PhysicalBackup {
+		typ = " physical restore.\nWaiting to finish"
+	}
+	fmt.Printf("Started%s", typ)
+	err = waitRestore(cn, m, pbm.StatusDone, tdiff)
+	if err == nil {
+		return restoreRet{
+			Name:     m.Name,
+			done:     true,
+			physical: m.Type == pbm.PhysicalBackup || m.Type == pbm.IncrementalBackup,
+		}, nil
+	}
+
+	if serr, ok := err.(errRestoreFailed); ok {
+		return restoreRet{err: serr.Error()}, nil
+	}
+	return restoreRet{err: fmt.Sprintf("%s.\n Try to check logs on node %s", err.Error(), m.Leader)}, nil
 }
 
 // We rely on heartbeats in error detection in case of all nodes failed,
@@ -145,7 +156,7 @@ func runRestore(cn *pbm.PBM, o *restoreOpts, outf outFormat) (fmt.Stringer, erro
 // But for physical ones, the cluster by this time is down. So we compare with
 // the wall time taking into account a time skew (wallTime - clusterTime) taken
 // when the cluster time was still available.
-func waitRestore(cn *pbm.PBM, m *pbm.RestoreMeta, tskew int64) error {
+func waitRestore(cn *pbm.PBM, m *pbm.RestoreMeta, status pbm.Status, tskew int64) error {
 	ep, _ := cn.GetEpoch()
 	l := cn.Logger().NewEvent(string(pbm.CmdRestore), m.Backup, m.OPID, ep.TS())
 	stg, err := cn.GetStorage(l)
@@ -181,7 +192,7 @@ func waitRestore(cn *pbm.PBM, m *pbm.RestoreMeta, tskew int64) error {
 		}
 
 		switch rmeta.Status {
-		case pbm.StatusDone, pbm.StatusPartlyDone:
+		case status, pbm.StatusDone, pbm.StatusPartlyDone:
 			return nil
 		case pbm.StatusError:
 			return errRestoreFailed{fmt.Sprintf("operation failed with: %s", rmeta.Error)}
@@ -213,33 +224,89 @@ func (e errRestoreFailed) Error() string {
 	return e.string
 }
 
-func restore(cn *pbm.PBM, bcpName string, nss []string, rsMapping map[string]string, outf outFormat) (*pbm.RestoreMeta, error) {
-	bcp, err := cn.GetBackupMeta(bcpName)
-	if errors.Is(err, pbm.ErrNotFound) {
-		return nil, errors.Errorf("backup '%s' not found", bcpName)
-	}
-	if err != nil {
-		return nil, errors.Wrap(err, "get backup data")
-	}
-	if bcp.Status != pbm.StatusDone {
-		return nil, errors.Errorf("backup '%s' didn't finish successfully", bcpName)
+func checkBackup(cn *pbm.PBM, o *restoreOpts, nss []string) (name string, typ pbm.BackupType, err error) {
+	if o.extern && o.bcp == "" {
+		return "", pbm.ExternalBackup, nil
 	}
 
+	if o.pitr != "" && o.pitrBase == "" {
+		return "", pbm.LogicalBackup, nil
+	}
+
+	b := o.bcp
+	if o.pitrBase != "" {
+		b = o.pitrBase
+	}
+
+	bcp, err := cn.GetBackupMeta(b)
+	if errors.Is(err, pbm.ErrNotFound) {
+		return "", "", errors.Errorf("backup '%s' not found", b)
+	}
+	if err != nil {
+		return "", "", errors.Wrap(err, "get backup data")
+	}
+	if len(nss) != 0 && bcp.Type != pbm.LogicalBackup {
+		return "", "", errors.New("--ns flag is only allowed for logical restore")
+	}
+	if bcp.Status != pbm.StatusDone {
+		return "", "", errors.Errorf("backup '%s' didn't finish successfully", b)
+	}
+
+	return b, bcp.Type, nil
+}
+
+func restore(cn *pbm.PBM, o *restoreOpts, nss []string, rsMapping map[string]string, outf outFormat) (*pbm.RestoreMeta, error) {
+	bcp, bcpType, err := checkBackup(cn, o, nss)
+	if err != nil {
+		return nil, err
+	}
 	err = checkConcurrentOp(cn)
 	if err != nil {
 		return nil, err
 	}
 
 	name := time.Now().UTC().Format(time.RFC3339Nano)
-	err = cn.SendCmd(pbm.Cmd{
+
+	cmd := pbm.Cmd{
 		Cmd: pbm.CmdRestore,
 		Restore: &pbm.RestoreCmd{
 			Name:       name,
-			BackupName: bcpName,
+			BackupName: bcp,
 			Namespaces: nss,
 			RSMap:      rsMapping,
+			External:   o.extern,
 		},
-	})
+	}
+	if o.pitr != "" {
+		cmd.Restore.OplogTS, err = parseTS(o.pitr)
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	if o.ts != "" {
+		cmd.Restore.ExtTS, err = parseTS(o.ts)
+		if err != nil {
+			return nil, err
+		}
+	}
+	if o.conf != "" {
+		var buf []byte
+		if o.conf == "-" {
+			buf, err = io.ReadAll(os.Stdin)
+		} else {
+			buf, err = os.ReadFile(o.conf)
+		}
+		if err != nil {
+			return nil, errors.Wrap(err, "unable to read config file")
+		}
+		err = yaml.UnmarshalStrict(buf, &cmd.Restore.ExtConf)
+		if err != nil {
+			return nil, errors.Wrap(err, "unable to  unmarshal config file")
+		}
+	}
+
+	err = cn.SendCmd(cmd)
 	if err != nil {
 		return nil, errors.Wrap(err, "send command")
 	}
@@ -247,12 +314,23 @@ func restore(cn *pbm.PBM, bcpName string, nss []string, rsMapping map[string]str
 	if outf != outText {
 		return &pbm.RestoreMeta{
 			Name:   name,
-			Backup: bcpName,
-			Type:   bcp.Type,
+			Backup: bcp,
+			Type:   bcpType,
 		}, nil
 	}
 
-	fmt.Printf("Starting restore %s from '%s'", name, bcpName)
+	bcpName := ""
+	if bcp != "" {
+		bcpName = fmt.Sprintf(" from '%s'", bcp)
+	}
+	if o.extern {
+		bcpName = " from [external]"
+	}
+	pitrs := ""
+	if o.pitr != "" {
+		pitrs = fmt.Sprintf(" to point-in-time %s", o.pitr)
+	}
+	fmt.Printf("Starting restore %s%s%s", name, pitrs, bcpName)
 
 	var (
 		fn     getRestoreMetaFn
@@ -262,26 +340,40 @@ func restore(cn *pbm.PBM, bcpName string, nss []string, rsMapping map[string]str
 
 	// physical restore may take more time to start
 	const waitPhysRestoreStart = time.Second * 120
-	if bcp.Type == pbm.PhysicalBackup || bcp.Type == pbm.IncrementalBackup {
+	if bcpType == pbm.LogicalBackup {
+		fn = cn.GetRestoreMeta
+		ctx, cancel = context.WithTimeout(context.Background(), pbm.WaitActionStart)
+	} else {
 		ep, _ := cn.GetEpoch()
-		stg, err := cn.GetStorage(cn.Logger().NewEvent(string(pbm.CmdRestore), bcpName, "", ep.TS()))
+		stg, err := cn.GetStorage(cn.Logger().NewEvent(string(pbm.CmdRestore), bcp, "", ep.TS()))
 		if err != nil {
 			return nil, errors.Wrap(err, "get storage")
 		}
 
 		fn = func(name string) (*pbm.RestoreMeta, error) {
-			return pbm.GetPhysRestoreMeta(name, stg, cn.Logger().NewEvent(string(pbm.CmdRestore), bcpName, "", ep.TS()))
+			return pbm.GetPhysRestoreMeta(name, stg, cn.Logger().NewEvent(string(pbm.CmdRestore), bcp, "", ep.TS()))
 		}
 		ctx, cancel = context.WithTimeout(context.Background(), waitPhysRestoreStart)
-	} else {
-		fn = cn.GetRestoreMeta
-		ctx, cancel = context.WithTimeout(context.Background(), pbm.WaitActionStart)
 	}
 	defer cancel()
 
 	return waitForRestoreStatus(ctx, cn, name, fn)
 }
 
+func runFinishRestore(o descrRestoreOpts) (fmt.Stringer, error) {
+	stg, err := getRestoreMetaStg(o.cfg)
+	if err != nil {
+		return nil, errors.Wrap(err, "get storage")
+	}
+
+	path := fmt.Sprintf("%s/%s/cluster", pbm.PhysRestoresDir, o.restore)
+	return outMsg{"Command sent. Check `pbm describe-restore ...` for the result."},
+		stg.Save(path+"."+string(pbm.StatusCopyDone),
+			bytes.NewReader([]byte(
+				fmt.Sprintf("%d", time.Now().Unix()),
+			)), -1)
+}
+
 func parseTS(t string) (ts primitive.Timestamp, err error) {
 	if si := strings.SplitN(t, ",", 2); len(si) == 2 {
 		tt, err := strconv.ParseInt(si[0], 10, 64)
@@ -304,45 +396,6 @@ func parseTS(t string) (ts primitive.Timestamp, err error) {
 	return primitive.Timestamp{T: uint32(tsto.Unix()), I: 0}, nil
 }
 
-func pitrestore(cn *pbm.PBM, t, base string, nss []string, rsMap map[string]string, outf outFormat) (rmeta *pbm.RestoreMeta, err error) {
-	ts, err := parseTS(t)
-	if err != nil {
-		return nil, err
-	}
-
-	err = checkConcurrentOp(cn)
-	if err != nil {
-		return nil, err
-	}
-
-	name := time.Now().UTC().Format(time.RFC3339Nano)
-	err = cn.SendCmd(pbm.Cmd{
-		Cmd: pbm.CmdPITRestore,
-		PITRestore: &pbm.PITRestoreCmd{
-			Name:       name,
-			TS:         int64(ts.T),
-			I:          int64(ts.I),
-			Bcp:        base,
-			Namespaces: nss,
-			RSMap:      rsMap,
-		},
-	})
-	if err != nil {
-		return nil, errors.Wrap(err, "send command")
-	}
-
-	if outf != outText {
-		return &pbm.RestoreMeta{Name: name}, nil
-	}
-
-	fmt.Printf("Starting restore to the point in time '%s'", t)
-
-	ctx, cancel := context.WithTimeout(context.Background(), pbm.WaitActionStart)
-	defer cancel()
-
-	return waitForRestoreStatus(ctx, cn, name, cn.GetRestoreMeta)
-}
-
 type getRestoreMetaFn func(name string) (*pbm.RestoreMeta, error)
 
 func waitForRestoreStatus(ctx context.Context, cn *pbm.PBM, name string, getfn getRestoreMetaFn) (*pbm.RestoreMeta, error) {
@@ -421,10 +474,12 @@ type describeRestoreResult struct {
 type RestoreReplset struct {
 	Name               string        `json:"name" yaml:"name"`
 	Status             pbm.Status    `json:"status" yaml:"status"`
-	Error              *string       `json:"error,omitempty" yaml:"error,omitempty"`
+	PartialTxn         []db.Oplog    `json:"partial_txn,omitempty" yaml:"-"`
+	PartialTxnStr      *string       `json:"-" yaml:"partial_txn,omitempty"`
 	LastTransitionTS   int64         `json:"last_transition_ts" yaml:"-"`
 	LastTransitionTime string        `json:"last_transition_time" yaml:"last_transition_time"`
 	Nodes              []RestoreNode `json:"nodes,omitempty" yaml:"nodes,omitempty"`
+	Error              *string       `json:"error,omitempty" yaml:"error,omitempty"`
 }
 
 type RestoreNode struct {
@@ -444,33 +499,38 @@ func (r describeRestoreResult) String() string {
 	return string(b)
 }
 
+func getRestoreMetaStg(cfgPath string) (storage.Storage, error) {
+	buf, err := ioutil.ReadFile(cfgPath)
+	if err != nil {
+		return nil, errors.Wrap(err, "unable to read config file")
+	}
+
+	var cfg pbm.Config
+	err = yaml.UnmarshalStrict(buf, &cfg)
+	if err != nil {
+		return nil, errors.Wrap(err, "unable to  unmarshal config file")
+	}
+
+	l := log.New(nil, "cli", "").NewEvent("", "", "", primitive.Timestamp{})
+	return pbm.Storage(cfg, l)
+}
+
 func describeRestore(cn *pbm.PBM, o descrRestoreOpts) (fmt.Stringer, error) {
-	var res describeRestoreResult
-	var meta *pbm.RestoreMeta
+	var (
+		meta *pbm.RestoreMeta
+		err  error
+		res  describeRestoreResult
+	)
 	if o.cfg != "" {
-		buf, err := ioutil.ReadFile(o.cfg)
-		if err != nil {
-			return nil, errors.Wrap(err, "unable to read config file")
-		}
-
-		var cfg pbm.Config
-		err = yaml.UnmarshalStrict(buf, &cfg)
-		if err != nil {
-			return nil, errors.Wrap(err, "unable to  unmarshal config file")
-		}
-
-		l := log.New(nil, "cli", "").NewEvent("", "", "", primitive.Timestamp{})
-		stg, err := pbm.Storage(cfg, l)
+		stg, err := getRestoreMetaStg(o.cfg)
 		if err != nil {
 			return nil, errors.Wrap(err, "get storage")
 		}
-
-		meta, err = pbm.GetPhysRestoreMeta(o.restore, stg, l)
+		meta, err = pbm.GetPhysRestoreMeta(o.restore, stg, log.New(nil, "cli", "").NewEvent("", "", "", primitive.Timestamp{}))
 		if err != nil && meta == nil {
 			return nil, errors.Wrap(err, "get restore meta")
 		}
 	} else {
-		var err error
 		meta, err = cn.GetRestoreMeta(o.restore)
 		if err != nil {
 			return nil, errors.Wrap(err, "get restore meta")
@@ -508,10 +568,20 @@ func describeRestore(cn *pbm.PBM, o descrRestoreOpts) (fmt.Stringer, error) {
 			Name:               rs.Name,
 			Status:             rs.Status,
 			LastTransitionTS:   rs.LastTransitionTS,
+			PartialTxn:         rs.PartialTxn,
 			LastTransitionTime: time.Unix(rs.LastTransitionTS, 0).UTC().Format(time.RFC3339),
 		}
 		if rs.Status == pbm.StatusError {
 			mrs.Error = &rs.Error
+		} else if len(mrs.PartialTxn) > 0 {
+			b, err := json.Marshal(mrs.PartialTxn)
+			if err != nil {
+				return res, errors.Wrap(err, "marshal partially committed transactions")
+			}
+			str := string(b)
+			mrs.PartialTxnStr = &str
+			perr := "WARNING! Some distributed transactions were not full in the oplog for this shard. But were applied on other shard(s). See the list of not applied ops in `partial_txn`."
+			mrs.Error = &perr
 		}
 		for _, node := range rs.Nodes {
 			mnode := RestoreNode{
