@@ -20,6 +20,7 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm"
 	plog "github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/pitr"
+	"github.com/percona/percona-backup-mongodb/pbm/sel"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 )
 
@@ -434,7 +435,7 @@ func (c currOp) String() string {
 	switch c.Type {
 	default:
 		return fmt.Sprintf("%s [op id: %s]", c.Type, c.OPID)
-	case pbm.CmdBackup, pbm.CmdRestore, pbm.CmdPITRestore:
+	case pbm.CmdBackup, pbm.CmdRestore:
 		return fmt.Sprintf("%s \"%s\", started at %s. Status: %s. [op id: %s]",
 			c.Type, c.Name, time.Unix((c.StartTS), 0).UTC().Format("2006-01-02T15:04:05Z"),
 			c.Status, c.OPID,
@@ -485,7 +486,7 @@ func getCurrOps(cn *pbm.PBM) (fmt.Stringer, error) {
 		case pbm.StatusDumpDone:
 			r.Status = "oplog backup"
 		}
-	case pbm.CmdRestore, pbm.CmdPITRestore:
+	case pbm.CmdRestore:
 		rst, err := cn.GetRestoreMetaByOPID(r.OPID)
 		if err != nil {
 			return r, errors.Wrap(err, "get restore info")
@@ -727,8 +728,9 @@ func getPITRranges(cn *pbm.PBM, stg storage.Storage, bcps []pbm.BackupMeta, rsMa
 	for _, tl := range pbm.MergeTimelines(rstlines...) {
 		var bcplastWrite *primitive.Timestamp
 
-		for _, bcp := range bcps {
-			if bcp.Error() != nil {
+		for i := range bcps {
+			bcp := &bcps[i]
+			if !isValidBaseSnapshot(bcp) {
 				continue
 			}
 
@@ -746,6 +748,28 @@ func getPITRranges(cn *pbm.PBM, stg storage.Storage, bcps []pbm.BackupMeta, rsMa
 	return &pitrRanges{Ranges: pr, Size: size}, nil
 }
 
+func isValidBaseSnapshot(bcp *pbm.BackupMeta) bool {
+	if bcp.Status != pbm.StatusDone || sel.IsSelective(bcp.Namespaces) {
+		return false
+	}
+
+	err := bcp.Error()
+	if err == nil {
+		return true
+	}
+
+	switch err.(type) {
+	case errMissedReplsets, errIncompatibleFCVVersion:
+		return true
+	case errIncompatibleMongodVersion:
+		if bcp.Type == pbm.LogicalBackup {
+			return true
+		}
+	}
+
+	return false
+}
+
 func getBackupSize(bcp *pbm.BackupMeta, stg storage.Storage) (s int64, err error) {
 	if bcp.Size > 0 {
 		return bcp.Size, nil
@@ -769,6 +793,8 @@ func getLegacySnapshotSize(bcp *pbm.BackupMeta, stg storage.Storage) (s int64, e
 		return getLegacyLogicalSize(bcp, stg)
 	case pbm.PhysicalBackup, pbm.IncrementalBackup:
 		return getLegacyPhysSize(bcp.Replsets, stg)
+	case pbm.ExternalBackup:
+		return 0, nil
 	default:
 		return 0, errors.Errorf("unknown backup type %s", bcp.Type)
 	}
