@@ -5,6 +5,9 @@ import (
 
 	"github.com/percona/percona-backup-mongodb/e2e-tests/pkg/tests/sharded"
 	"github.com/percona/percona-backup-mongodb/pbm"
+
+	"math/rand"
+	"time"
 )
 
 func runPhysical(t *sharded.Cluster, typ testTyp) {
@@ -22,9 +25,19 @@ func runPhysical(t *sharded.Cluster, typ testTyp) {
 		{"GCS", "/etc/pbm/gcs.yaml"},
 		{"Azure", "/etc/pbm/azure.yaml"},
 		{"FS", "/etc/pbm/fs.yaml"},
-		{"Minio", "/etc/pbm/minio.yaml"},
 	}
 
+	rand.Seed(time.Now().UnixNano())
+	rand.Shuffle(len(remoteStg), func(i, j int) {
+		remoteStg[i], remoteStg[j] = remoteStg[j], remoteStg[i]
+	})
+
+	minio := struct {
+		name string
+		conf string
+	}{name: "Minio", conf: "/etc/pbm/minio.yaml"}
+	remoteStg = append(remoteStg, minio)
+
 	for _, stg := range remoteStg {
 		if confExt(stg.conf) {
 			storage = stg.conf
