@@ -301,7 +301,7 @@ func (c *Ctl) RunCmd(cmds ...string) (string, error) {
 	}
 	defer container.Close()
 
-	tmr := time.NewTimer(pbm.WaitBackupStart)
+	tmr := time.NewTimer(time.Duration(float64(pbm.WaitBackupStart) * 1.5))
 	tkr := time.NewTicker(500 * time.Millisecond)
 	for {
 		select {
