@@ -32,6 +32,7 @@ type Backup struct {
 	node     *pbm.Node
 	typ      pbm.BackupType
 	incrBase bool
+	timeouts *pbm.BackupTimeouts
 }
 
 func New(cn *pbm.PBM, node *pbm.Node) *Backup {
@@ -67,7 +68,11 @@ func NewIncremental(cn *pbm.PBM, node *pbm.Node, base bool) *Backup {
 	}
 }
 
-func (b *Backup) Init(bcp *pbm.BackupCmd, opid pbm.OPID, inf *pbm.NodeInfo, balancer pbm.BalancerMode) error {
+func (b *Backup) SetTimeouts(t *pbm.BackupTimeouts) {
+	b.timeouts = t
+}
+
+func (b *Backup) Init(bcp *pbm.BackupCmd, opid pbm.OPID, inf *pbm.NodeInfo, store pbm.StorageConf, balancer pbm.BalancerMode) error {
 	ts, err := b.cn.ClusterTime()
 	if err != nil {
 		return errors.Wrap(err, "read cluster time")
@@ -79,6 +84,7 @@ func (b *Backup) Init(bcp *pbm.BackupCmd, opid pbm.OPID, inf *pbm.NodeInfo, bala
 		Name:           bcp.Name,
 		Namespaces:     bcp.Namespaces,
 		Compression:    bcp.Compression,
+		Store:          store,
 		StartTS:        time.Now().Unix(),
 		Status:         pbm.StatusStarting,
 		Replsets:       []pbm.BackupReplset{},
@@ -90,14 +96,6 @@ func (b *Backup) Init(bcp *pbm.BackupCmd, opid pbm.OPID, inf *pbm.NodeInfo, bala
 		Hb:             ts,
 	}
 
-	cfg, err := b.cn.GetConfig()
-	if err == pbm.ErrStorageUndefined {
-		return errors.New("backups cannot be saved because PBM storage configuration hasn't been set yet")
-	} else if err != nil {
-		return errors.Wrap(err, "unable to get PBM config settings")
-	}
-	meta.Store = cfg.Storage
-
 	ver, err := b.node.GetMongoVersion()
 	if err != nil {
 		return errors.WithMessage(err, "get mongo version")
@@ -232,7 +230,7 @@ func (b *Backup) Run(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPID, l *
 
 	// Waiting for StatusStarting to move further.
 	// In case some preparations has to be done before backup.
-	err = b.waitForStatus(bcp.Name, pbm.StatusStarting, &pbm.WaitBackupStart)
+	err = b.waitForStatus(bcp.Name, pbm.StatusStarting, ref(b.timeouts.StartingStatus()))
 	if err != nil {
 		return errors.Wrap(err, "waiting for start")
 	}
@@ -682,3 +680,7 @@ func (b *Backup) setClusterLastWrite(bcpName string) error {
 	err = b.cn.SetLastWrite(bcpName, lw)
 	return errors.Wrap(err, "set timestamp")
 }
+
+func ref[T any](v T) *T {
+	return &v
+}
