@@ -269,6 +269,8 @@ type MongodOptsSec struct {
 	} `bson:"kmip,omitempty" json:"kmip,omitempty" yaml:"kmip,omitempty"`
 }
 
+type ExternOpts map[string]MongodOpts
+
 type MongodOptsStorage struct {
 	DirectoryPerDB bool   `bson:"directoryPerDB" json:"directoryPerDB" yaml:"directoryPerDB"`
 	DBpath         string `bson:"dbPath" json:"dbPath" yaml:"dbPath"`
@@ -286,6 +288,26 @@ type MongodOptsStorage struct {
 	} `bson:"wiredTiger" json:"wiredTiger" yaml:"wiredTiger"`
 }
 
+// NewMongodOptsStorage return MongodOptsStorage with default settings
+func NewMongodOptsStorage() *MongodOptsStorage {
+	m := &MongodOptsStorage{}
+	m.setDefaults()
+	return m
+}
+
+func (stg *MongodOptsStorage) setDefaults() {
+	stg.DBpath = "/data/db"
+	stg.WiredTiger.EngineConfig.JournalCompressor = "snappy"
+	stg.WiredTiger.CollectionConfig.BlockCompressor = "snappy"
+	stg.WiredTiger.IndexConfig.PrefixCompression = true
+}
+
+func (stg *MongodOptsStorage) UnmarshalYAML(unmarshal func(interface{}) error) error {
+	stg.setDefaults()
+	type rawStg MongodOptsStorage
+	return unmarshal((*rawStg)(stg))
+}
+
 type RSConfig struct {
 	ID                      string     `bson:"_id" json:"_id"`
 	CSRS                    bool       `bson:"configsvr,omitempty" json:"configsvr"`
