@@ -32,21 +32,26 @@ type OplogChunk struct {
 	Compression compress.CompressionType `bson:"compression"`
 	StartTS     primitive.Timestamp      `bson:"start_ts"`
 	EndTS       primitive.Timestamp      `bson:"end_ts"`
-	size        int64                    `bson:"-"`
+	Size        int64                    `bson:"size"`
 }
 
 // IsPITR checks if PITR is enabled
 func (p *PBM) IsPITR() (bool, error) {
-	cfg, err := p.GetConfig()
+	enabled, _, err := isPITREnabled(p.ctx, p.Conn)
+	return enabled, err
+}
+
+func isPITREnabled(ctx context.Context, m *mongo.Client) (bool, bool, error) {
+	cfg, err := getPBMConfig(ctx, m)
 	if err != nil {
 		if errors.Is(err, mongo.ErrNoDocuments) {
-			return false, nil
+			return false, false, nil
 		}
 
-		return false, errors.Wrap(err, "get config")
+		return false, false, errors.Wrap(err, "get config")
 	}
 
-	return cfg.PITR.Enabled, nil
+	return cfg.PITR.Enabled, cfg.PITR.OplogOnly, nil
 }
 
 // PITRrun checks if PITR slicing is running. It looks for PITR locks
@@ -225,7 +230,7 @@ func (t Timeline) String() string {
 // any saved chunk already belongs to some valid timeline,
 // the slice wouldn't be done otherwise.
 // `flist` is a cache of chunk sizes.
-func (p *PBM) PITRGetValidTimelines(rs string, until primitive.Timestamp, flist map[string]int64) (tlines []Timeline, err error) {
+func (p *PBM) PITRGetValidTimelines(rs string, until primitive.Timestamp) (tlines []Timeline, err error) {
 	fch, err := p.PITRFirstChunkMeta(rs)
 	if err != nil {
 		return nil, errors.Wrap(err, "get the oldest chunk")
@@ -239,12 +244,6 @@ func (p *PBM) PITRGetValidTimelines(rs string, until primitive.Timestamp, flist
 		return nil, errors.Wrap(err, "get slice")
 	}
 
-	if flist != nil {
-		for i, s := range slices {
-			slices[i].size = flist[s.FName]
-		}
-	}
-
 	return gettimelines(slices), nil
 }
 
@@ -262,7 +261,7 @@ func (p *PBM) PITRTimelines() (tlines []Timeline, err error) {
 
 	var tlns [][]Timeline
 	for _, s := range shards {
-		t, err := p.PITRGetValidTimelines(s.RS, now, nil)
+		t, err := p.PITRGetValidTimelines(s.RS, now)
 		if err != nil {
 			return nil, errors.Wrapf(err, "get PITR timelines for %s replset", s.RS)
 		}
@@ -287,7 +286,7 @@ func gettimelines(slices []OplogChunk) (tlines []Timeline) {
 		}
 		prevEnd = s.EndTS
 		tl.End = s.EndTS.T
-		tl.Size += s.size
+		tl.Size += s.Size
 	}
 
 	tlines = append(tlines, tl)
